@@ -3,12 +3,11 @@
 import pytest
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_linear_gradient():
-    assert_pixels('linear_gradient', 10, 10, '''
+def test_linear_gradient(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         BBBBBBBBBB
@@ -38,8 +37,8 @@ def test_linear_gradient():
 
 
 @assert_no_logs
-def test_linear_gradient_userspace():
-    assert_pixels('linear_gradient_userspace', 10, 10, '''
+def test_linear_gradient_userspace(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         BBBBBBBBBB
@@ -69,8 +68,8 @@ def test_linear_gradient_userspace():
 
 
 @assert_no_logs
-def test_linear_gradient_multicolor():
-    assert_pixels('linear_gradient_multicolor', 10, 8, '''
+def test_linear_gradient_multicolor(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         RRRRRRRRRR
@@ -102,8 +101,8 @@ def test_linear_gradient_multicolor():
 
 
 @assert_no_logs
-def test_linear_gradient_multicolor_userspace():
-    assert_pixels('linear_gradient_multicolor_userspace', 10, 8, '''
+def test_linear_gradient_multicolor_userspace(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         RRRRRRRRRR
@@ -136,8 +135,8 @@ def test_linear_gradient_multicolor_userspace():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_linear_gradient_transform():
-    assert_pixels('linear_gradient_transform', 10, 8, '''
+def test_linear_gradient_transform(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         RRRRRRRRRR
         GGGGGGGGGG
@@ -169,8 +168,8 @@ def test_linear_gradient_transform():
 
 
 @assert_no_logs
-def test_linear_gradient_repeat():
-    assert_pixels('linear_gradient_repeat', 10, 16, '''
+def test_linear_gradient_repeat(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         RRRRRRRRRR
@@ -211,8 +210,8 @@ def test_linear_gradient_repeat():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_linear_gradient_repeat_long():
-    assert_pixels('linear_gradient_repeat_long', 10, 16, '''
+def test_linear_gradient_repeat_long(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         RRRRRRRRRR
         GGGGGGGGGG
@@ -252,8 +251,8 @@ def test_linear_gradient_repeat_long():
 
 
 @assert_no_logs
-def test_linear_gradient_reflect():
-    assert_pixels('linear_gradient_reflect', 10, 16, '''
+def test_linear_gradient_reflect(assert_pixels):
+    assert_pixels('''
         BBBBBBBBBB
         BBBBBBBBBB
         RRRRRRRRRR
@@ -293,8 +292,8 @@ def test_linear_gradient_reflect():
 
 
 @assert_no_logs
-def test_radial_gradient():
-    assert_pixels('radial_gradient', 10, 10, '''
+def test_radial_gradient(assert_pixels):
+    assert_pixels('''
         rrrrrrrrrr
         rrrrrrrrrr
         rrrrBBrrrr
@@ -325,8 +324,8 @@ def test_radial_gradient():
 
 
 @assert_no_logs
-def test_radial_gradient_userspace():
-    assert_pixels('radial_gradient_userspace', 10, 10, '''
+def test_radial_gradient_userspace(assert_pixels):
+    assert_pixels('''
         rrrrrrrrrr
         rrrrrrrrrr
         rrrrBBrrrr
@@ -356,8 +355,8 @@ def test_radial_gradient_userspace():
 
 
 @assert_no_logs
-def test_radial_gradient_multicolor():
-    assert_pixels('radial_gradient_multicolor', 10, 10, '''
+def test_radial_gradient_multicolor(assert_pixels):
+    assert_pixels('''
         rrrrrrrrrr
         rrrGGGGrrr
         rrGGBBGGrr
@@ -390,8 +389,8 @@ def test_radial_gradient_multicolor():
 
 
 @assert_no_logs
-def test_radial_gradient_multicolor_userspace():
-    assert_pixels('radial_gradient_multicolor_userspace', 10, 10, '''
+def test_radial_gradient_multicolor_userspace(assert_pixels):
+    assert_pixels('''
         rrrrrrrrrr
         rrrGGGGrrr
         rrGGBBGGrr
@@ -425,8 +424,8 @@ def test_radial_gradient_multicolor_userspace():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_radial_gradient_repeat():
-    assert_pixels('radial_gradient_repeat', 10, 10, '''
+def test_radial_gradient_repeat(assert_pixels):
+    assert_pixels('''
         GBrrrrrrBG
         BrrGGGGrrB
         rrGGBBGGrr
@@ -460,8 +459,8 @@ def test_radial_gradient_repeat():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_radial_gradient_reflect():
-    assert_pixels('radial_gradient_reflect', 10, 10, '''
+def test_radial_gradient_reflect(assert_pixels):
+    assert_pixels('''
         BGrrrrrrGB
         GrrGGGGrrG
         rrGGBBGGrr
@@ -491,3 +490,34 @@ def test_radial_gradient_reflect():
         <rect x="0" y="0" width="10" height="10" fill="url(#grad)" />
       </svg>
     ''')
+
+
+@assert_no_logs
+def test_linear_gradient_opacity(assert_pixels):
+    assert_pixels('''
+        BBBBBBBBBB
+        BBBBBBBBBB
+        BBBBBBBBBB
+        BBBBBBBBBB
+        BBBBBBBBBB
+        ssssssssss
+        ssssssssss
+        ssssssssss
+        ssssssssss
+        ssssssssss
+    ''', '''
+      <style>
+        @page { size: 10px }
+        svg { display: block }
+      </style>
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
+        <defs>
+          <linearGradient id="grad" x1="0" y1="0" x2="0" y2="1"
+            gradientUnits="objectBoundingBox">
+            <stop stop-color="blue" offset="50%"></stop>
+            <stop stop-color="red" stop-opacity="0.502" offset="50%"></stop>
+          </linearGradient>
+        </defs>
+        <rect x="0" y="0" width="10" height="10" fill="url(#grad)" />
+      </svg>
+    ''')
