@@ -1,10 +1,4 @@
-"""
-    weasyprint.tests.test_draw.test_box
-    -----------------------------------
-
-    Test how boxes, borders, outlines are drawn.
-
-"""
+"""Test how boxes, borders, outlines are drawn."""
 
 import itertools
 
@@ -12,16 +6,15 @@ import pytest
 from weasyprint import HTML
 
 from ..testing_utils import assert_no_logs
-from . import PIXELS_BY_CHAR, assert_different_renderings, assert_pixels
 
 
 @assert_no_logs
-def test_borders(margin='10px', prop='border'):
+def test_borders(assert_pixels, assert_different_renderings, margin='10px',
+                 prop='border'):
     """Test the rendering of borders"""
     source = '''
       <style>
         @page { size: 140px 110px }
-        html { background: #fff }
         body { width: 100px; height: 70px;
                margin: %s; %s: 10px %s blue }
       </style>
@@ -29,36 +22,39 @@ def test_borders(margin='10px', prop='border'):
 
     # Do not test the exact rendering of earch border style but at least
     # check that they do not do the same.
-    assert_different_renderings(140, 110, [
-        (f'{prop}_{border_style}', source % (margin, prop, border_style))
-        for border_style in [
+    documents = (
+        source % (margin, prop, border_style)
+        for border_style in (
             'none', 'solid', 'dashed', 'dotted', 'double',
-            'inset', 'outset', 'groove', 'ridge']])
+            'inset', 'outset', 'groove', 'ridge'))
+    assert_different_renderings(*documents)
 
     css_margin = margin
     width = 140
     height = 110
     margin = 10
     border = 10
-    solid_pixels = [PIXELS_BY_CHAR['_'] for i in range(width * height)]
+    solid_pixels = [['_'] * width for _ in range(height)]
     for x in range(margin, width - margin):
         for y in itertools.chain(
                 range(margin, margin + border),
                 range(height - margin - border, height - margin)):
-            solid_pixels[y * width + x] = PIXELS_BY_CHAR['B']
+            solid_pixels[y][x] = 'B'
     for y in range(margin, height - margin):
         for x in itertools.chain(
                 range(margin, margin + border),
                 range(width - margin - border, width - margin)):
-            solid_pixels[y * width + x] = PIXELS_BY_CHAR['B']
-    assert_pixels(
-        f'{prop}_solid', 140, 110, solid_pixels,
-        source % (css_margin, prop, 'solid'))
+            solid_pixels[y][x] = 'B'
+    pixels = '\n'.join(''.join(chars) for chars in solid_pixels)
+    html = source % (css_margin, prop, 'solid')
+    assert_pixels(pixels, html)
 
 
 @assert_no_logs
-def test_outlines():
-    return test_borders(margin='20px', prop='outline')
+def test_outlines(assert_pixels, assert_different_renderings):
+    return test_borders(
+        assert_pixels, assert_different_renderings,
+        margin='20px', prop='outline')
 
 
 @assert_no_logs
@@ -70,7 +66,6 @@ def test_small_borders_1(border_style):
     html = '''
       <style>
         @page { size: 50px 50px }
-        html { background: #fff }
         body { margin: 5px; height: 0; border: 10px %s blue }
       </style>
       <body>''' % border_style
@@ -86,7 +81,6 @@ def test_small_borders_2(border_style):
     html = '''
       <style>
         @page { size: 50px 50px }
-        html { background: #fff }
         body { height: 0; width: 0; border-width: 1px 0; border-style: %s }
       </style>
       <body>''' % border_style
@@ -101,8 +95,8 @@ def test_em_borders():
 
 
 @assert_no_logs
-def test_margin_boxes():
-    assert_pixels('margin_boxes', 15, 15, '''
+def test_margin_boxes(assert_pixels):
+    assert_pixels('''
         _______________
         _GGG______BBBB_
         _GGG______BBBB_
@@ -125,7 +119,6 @@ def test_margin_boxes():
         @page {
           size: 15px;
           margin: 4px 6px 7px 5px;
-          background: white;
 
           @top-left-corner {
             margin: 1px;
@@ -159,3 +152,76 @@ def test_display_inline_block_twice():
     html = '<div style="background: red; display: inline-block">'
     document = HTML(string=html).render()
     assert document.write_pdf() == document.write_pdf()
+
+
+@assert_no_logs
+def test_draw_border_radius(assert_pixels):
+    assert_pixels('''
+        ___zzzzz
+        __zzzzzz
+        _zzzzzzz
+        zzzzzzzz
+        zzzzzzzz
+        zzzzzzzR
+        zzzzzzRR
+        zzzzzRRR
+    ''', '''
+      <style>
+        @page {
+          size: 8px 8px;
+        }
+        div {
+          background: red;
+          border-radius: 50% 0 0 0;
+          height: 16px;
+          width: 16px;
+        }
+      </style>
+      <div></div>
+    ''')
+
+
+@assert_no_logs
+def test_draw_split_border_radius(assert_pixels):
+    assert_pixels('''
+        ___zzzzz
+        __zzzzzz
+        _zzzzzzz
+        zzzzzzzz
+        zzzzzzzz
+        zzzzzzzz
+        zzzzzzRR
+        zzzzzRRR
+
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+        RRRRRRRR
+
+        zzzzzzRR
+        zzzzzzzR
+        zzzzzzzz
+        zzzzzzzz
+        zzzzzzzz
+        zzzzzzzz
+        _zzzzzzz
+        __zzzzzz
+    ''', '''
+      <style>
+        @page {
+          size: 8px 8px;
+        }
+        div {
+          background: red;
+          color: transparent;
+          border-radius: 8px;
+          line-height: 9px;
+          width: 16px;
+        }
+      </style>
+      <div>a b c</div>
+    ''')
