@@ -1,20 +1,13 @@
-"""
-    weasyprint.tests.test_draw.test_float
-    -------------------------------------
-
-    Test how floats are drawn.
-
-"""
+"""Test how floats are drawn."""
 
 import pytest
 
 from ..testing_utils import assert_no_logs
-from . import assert_pixels
 
 
 @assert_no_logs
-def test_float():
-    assert_pixels('float', 10, 5, '''
+def test_float(assert_pixels):
+    assert_pixels('''
         rBBB__aaaa
         BBBB__aaaa
         BBBB__aaaa
@@ -22,7 +15,7 @@ def test_float():
         __________
     ''', '''
       <style>
-        @page { size: 10px 5px; background: white }
+        @page { size: 10px 5px }
       </style>
       <div>
         <img style="float: left" src="pattern.png">
@@ -32,8 +25,8 @@ def test_float():
 
 
 @assert_no_logs
-def test_float_rtl():
-    assert_pixels('float_rtl', 10, 5, '''
+def test_float_rtl(assert_pixels):
+    assert_pixels('''
         rBBB__aaaa
         BBBB__aaaa
         BBBB__aaaa
@@ -41,7 +34,7 @@ def test_float_rtl():
         __________
     ''', '''
       <style>
-        @page { size: 10px 5px; background: white }
+        @page { size: 10px 5px }
       </style>
       <div style="direction: rtl">
         <img style="float: left" src="pattern.png">
@@ -51,8 +44,8 @@ def test_float_rtl():
 
 
 @assert_no_logs
-def test_float_inline():
-    assert_pixels('float_inline', 15, 5, '''
+def test_float_inline(assert_pixels):
+    assert_pixels('''
         rBBBGG_____aaaa
         BBBBGG_____aaaa
         BBBB_______aaaa
@@ -61,7 +54,7 @@ def test_float_inline():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -74,8 +67,8 @@ def test_float_inline():
 
 
 @assert_no_logs
-def test_float_inline_rtl():
-    assert_pixels('float_inline_rtl', 15, 5, '''
+def test_float_inline_rtl(assert_pixels):
+    assert_pixels('''
         rBBB_____GGaaaa
         BBBB_____GGaaaa
         BBBB_______aaaa
@@ -84,7 +77,7 @@ def test_float_inline_rtl():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -97,8 +90,8 @@ def test_float_inline_rtl():
 
 
 @assert_no_logs
-def test_float_inline_block():
-    assert_pixels('float_inline_block', 15, 5, '''
+def test_float_inline_block(assert_pixels):
+    assert_pixels('''
         rBBBGG_____aaaa
         BBBBGG_____aaaa
         BBBB_______aaaa
@@ -107,7 +100,7 @@ def test_float_inline_block():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -120,8 +113,8 @@ def test_float_inline_block():
 
 
 @assert_no_logs
-def test_float_inline_block_rtl():
-    assert_pixels('float_inline_block_rtl', 15, 5, '''
+def test_float_inline_block_rtl(assert_pixels):
+    assert_pixels('''
         rBBB_____GGaaaa
         BBBB_____GGaaaa
         BBBB_______aaaa
@@ -130,7 +123,7 @@ def test_float_inline_block_rtl():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -143,8 +136,8 @@ def test_float_inline_block_rtl():
 
 
 @assert_no_logs
-def test_float_table():
-    assert_pixels('float_table', 15, 5, '''
+def test_float_table(assert_pixels):
+    assert_pixels('''
         rBBBGG_____aaaa
         BBBBGG_____aaaa
         BBBB_______aaaa
@@ -153,7 +146,7 @@ def test_float_table():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -166,8 +159,8 @@ def test_float_table():
 
 
 @assert_no_logs
-def test_float_table_rtl():
-    assert_pixels('float_table_rtl', 15, 5, '''
+def test_float_table_rtl(assert_pixels):
+    assert_pixels('''
         rBBB_____GGaaaa
         BBBB_____GGaaaa
         BBBB_______aaaa
@@ -176,7 +169,7 @@ def test_float_table_rtl():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -189,8 +182,8 @@ def test_float_table_rtl():
 
 
 @assert_no_logs
-def test_float_inline_table():
-    assert_pixels('float_inline_table', 15, 5, '''
+def test_float_inline_table(assert_pixels):
+    assert_pixels('''
         rBBBGG_____aaaa
         BBBBGG_____aaaa
         BBBB_______aaaa
@@ -199,7 +192,7 @@ def test_float_inline_table():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         table { display: inline-table }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
@@ -213,8 +206,8 @@ def test_float_inline_table():
 
 
 @assert_no_logs
-def test_float_inline_table_rtl():
-    assert_pixels('float_inline_table_rtl', 15, 5, '''
+def test_float_inline_table_rtl(assert_pixels):
+    assert_pixels('''
         rBBB_____GGaaaa
         BBBB_____GGaaaa
         BBBB_______aaaa
@@ -223,7 +216,7 @@ def test_float_inline_table_rtl():
     ''', '''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         table { display: inline-table }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
@@ -237,8 +230,8 @@ def test_float_inline_table_rtl():
 
 
 @assert_no_logs
-def test_float_replaced_block():
-    assert_pixels('float_replaced_block', 15, 5, '''
+def test_float_replaced_block(assert_pixels):
+    assert_pixels('''
         rBBBaaaa___rBBB
         BBBBaaaa___BBBB
         BBBBaaaa___BBBB
@@ -246,7 +239,7 @@ def test_float_replaced_block():
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
       </style>
       <div>
         <img style="float: left" src="pattern.png">
@@ -257,8 +250,8 @@ def test_float_replaced_block():
 
 
 @assert_no_logs
-def test_float_replaced_block_rtl():
-    assert_pixels('float_replaced_block_rtl', 15, 5, '''
+def test_float_replaced_block_rtl(assert_pixels):
+    assert_pixels('''
         rBBB___aaaarBBB
         BBBB___aaaaBBBB
         BBBB___aaaaBBBB
@@ -266,7 +259,7 @@ def test_float_replaced_block_rtl():
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
       </style>
       <div style="direction: rtl">
         <img style="float: left" src="pattern.png">
@@ -278,8 +271,8 @@ def test_float_replaced_block_rtl():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_float_replaced_inline():
-    assert_pixels('float_replaced_inline', 15, 5, '''
+def test_float_replaced_inline(assert_pixels):
+    assert_pixels('''
         rBBBaaaa___rBBB
         BBBBaaaa___BBBB
         BBBBaaaa___BBBB
@@ -287,7 +280,7 @@ def test_float_replaced_inline():
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { line-height: 1px }
       </style>
       <div>
@@ -300,8 +293,8 @@ def test_float_replaced_inline():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_float_replaced_inline_rtl():
-    assert_pixels('float_replaced_inline_rtl', 15, 5, '''
+def test_float_replaced_inline_rtl(assert_pixels):
+    assert_pixels('''
         rBBB___aaaarBBB
         BBBB___aaaaBBBB
         BBBB___aaaaBBBB
@@ -309,7 +302,7 @@ def test_float_replaced_inline_rtl():
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px; background: white }
+        @page { size: 15px 5px }
         body { line-height: 1px }
       </style>
       <div style="direction: rtl">
@@ -321,18 +314,16 @@ def test_float_replaced_inline_rtl():
 
 
 @assert_no_logs
-def test_float_margin():
-    expected_pixels = '''
+def test_float_margin(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRRR__
         BBBBRRRRRRRRRR__
         __RRRRRRRRRR____
         __RRRRRRRRRR____
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -340,8 +331,6 @@ def test_float_margin():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -353,24 +342,20 @@ def test_float_margin():
             }
         </style>
         <div class="split">aa</div>
-        <div class="pushed">bbbbb bbbbb</div>
-    '''
-    assert_pixels('float_split_10', 16, 4, expected_pixels, html)
+        <div class="pushed">bbbbb bbbbb</div>''')
 
 
 @assert_no_logs
-def test_float_split_1():
-    expected_pixels = '''
+def test_float_split_1(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRRRRR
         BBBBRRRRRRRRRRRR
         BBBBRRRR________
         BBBBRRRR________
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -378,8 +363,6 @@ def test_float_split_1():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -388,24 +371,20 @@ def test_float_split_1():
             }
         </style>
         <div class="split">aa aa</div>
-        <div>bbbbbb bb</div>
-    '''
-    assert_pixels('float_split_1', 16, 4, expected_pixels, html)
+        <div>bbbbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_2():
-    expected_pixels = '''
+def test_float_split_2(assert_pixels):
+    assert_pixels('''
         RRRRRRRRRRRRBBBB
         RRRRRRRRRRRRBBBB
         RRRR________BBBB
         RRRR________BBBB
-    '''
-    html = '''
+    ''', '''
         <style>
           @font-face {src: url(weasyprint.otf); font-family: weasyprint}
           @page {
-            background: white;
             size: 16px 2px;
           }
           body {
@@ -413,8 +392,6 @@ def test_float_split_2():
             font-family: weasyprint;
             font-size: 2px;
             line-height: 1;
-            orphans: 1;
-            widows: 1;
           }
           div.split {
             color: blue;
@@ -423,24 +400,20 @@ def test_float_split_2():
           }
         </style>
         <div class="split">aa aa</div>
-        <div>bbbbbb bb</div>
-    '''
-    assert_pixels('float_split_2', 16, 4, expected_pixels, html)
+        <div>bbbbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_3():
-    expected_pixels = '''
+def test_float_split_3(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRRRRR
         BBBBRRRRRRRRRRRR
         RRRRRRRRRR______
         RRRRRRRRRR______
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -448,8 +421,6 @@ def test_float_split_3():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -458,24 +429,20 @@ def test_float_split_3():
             }
         </style>
         <div class="split">aa</div>
-        <div>bbbbbb bbbbb</div>
-    '''
-    assert_pixels('float_split_3', 16, 4, expected_pixels, html)
+        <div>bbbbbb bbbbb</div>''')
 
 
 @assert_no_logs
-def test_float_split_4():
-    expected_pixels = '''
+def test_float_split_4(assert_pixels):
+    assert_pixels('''
         RRRRRRRRRRRRBBBB
         RRRRRRRRRRRRBBBB
         RRRRRRRRRR______
         RRRRRRRRRR______
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -483,8 +450,6 @@ def test_float_split_4():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -493,24 +458,20 @@ def test_float_split_4():
             }
         </style>
         <div class="split">aa</div>
-        <div>bbbbbb bbbbb</div>
-    '''
-    assert_pixels('float_split_4', 16, 4, expected_pixels, html)
+        <div>bbbbbb bbbbb</div>''')
 
 
 @assert_no_logs
-def test_float_split_5():
-    expected_pixels = '''
+def test_float_split_5(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRgggg
         BBBBRRRRRRRRgggg
         BBBBRRRR____gggg
         BBBBRRRR____gggg
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -518,8 +479,6 @@ def test_float_split_5():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -533,24 +492,20 @@ def test_float_split_5():
         </style>
         <div class="split">aa aa</div>
         <div class="split2">cc cc</div>
-        <div>bbbb bb</div>
-    '''
-    assert_pixels('float_split_5', 16, 4, expected_pixels, html)
+        <div>bbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_6():
-    expected_pixels = '''
+def test_float_split_6(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRgggg
         BBBBRRRRRRRRgggg
         BBBBRRRR________
         BBBBRRRR________
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -558,8 +513,6 @@ def test_float_split_6():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -573,24 +526,20 @@ def test_float_split_6():
         </style>
         <div class="split">aa aa</div>
         <div class="split2">cc</div>
-        <div>bbbb bb</div>
-    '''
-    assert_pixels('float_split_6', 16, 4, expected_pixels, html)
+        <div>bbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_7():
-    expected_pixels = '''
+def test_float_split_7(assert_pixels):
+    assert_pixels('''
         BBBBRRRRRRRRgggg
         BBBBRRRRRRRRgggg
         RRRR________gggg
         RRRR________gggg
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -598,8 +547,6 @@ def test_float_split_7():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -613,24 +560,20 @@ def test_float_split_7():
         </style>
         <div class="split">aa</div>
         <div class="split2">cc cc</div>
-        <div>bbbb bb</div>
-    '''
-    assert_pixels('float_split_7', 16, 4, expected_pixels, html)
+        <div>bbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_8():
-    expected_pixels = '''
+def test_float_split_8(assert_pixels):
+    assert_pixels('''
         BBBB__RRRRRRRRRR
         BBBB__RRRRRRRRRR
         BBBB__RRRR______
         BBBB__RRRR______
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -638,8 +581,6 @@ def test_float_split_8():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -649,24 +590,20 @@ def test_float_split_8():
             }
         </style>
         <div class="split">aa aa</div>
-        <div>bbbbb bb</div>
-    '''
-    assert_pixels('float_split_8', 16, 4, expected_pixels, html)
+        <div>bbbbb bb</div>''')
 
 
 @assert_no_logs
-def test_float_split_9():
-    expected_pixels = '''
+def test_float_split_9(assert_pixels):
+    assert_pixels('''
         RRRRRRRRRRBBBB__
         RRRRRRRRRRBBBB__
         RRRR______BBBB__
         RRRR______BBBB__
-    '''
-    html = '''
+    ''', '''
         <style>
             @font-face {src: url(weasyprint.otf); font-family: weasyprint}
             @page {
-                background: white;
                 size: 16px 2px;
             }
             body {
@@ -674,8 +611,6 @@ def test_float_split_9():
                 font-family: weasyprint;
                 font-size: 2px;
                 line-height: 1;
-                orphans: 1;
-                widows: 1;
             }
             div.split {
                 color: blue;
@@ -685,6 +620,81 @@ def test_float_split_9():
             }
         </style>
         <div class="split">aa aa</div>
-        <div>bbbbb bb</div>
-    '''
-    assert_pixels('float_split_9', 16, 4, expected_pixels, html)
+        <div>bbbbb bb</div>''')
+
+
+@pytest.mark.xfail
+@assert_no_logs
+def test_float_split_10(assert_pixels):
+    assert_pixels('''
+        RRRRRRRRRR______
+        RRRRRRRRRR______
+        RRRRRRRRRR______
+        RRRRRRRRRR______
+        ________________
+        RRRRRR____BBBB__
+        RRRRRR____BBBB__
+        RRRRRR____BBBB__
+        RRRRRR____BBBB__
+        ________________
+    ''', '''
+        <style>
+            @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+            @page {
+                size: 16px 5px;
+            }
+            body {
+                color: red;
+                font-family: weasyprint;
+                font-size: 2px;
+                line-height: 1;
+            }
+            div.split {
+                color: blue;
+                float: right;
+                margin-right: 2px;
+                width: 4px;
+            }
+        </style>
+        <div>bbbbb bbbbb</div>
+        <div class="split">aa aa</div>
+        <div>bbb bbb</div>''')
+
+
+@assert_no_logs
+def test_float_split_11(assert_pixels):
+    assert_pixels('''
+        ________________
+        _BBBBBBBBBB_____
+        _BBBBBBBBBB_____
+        _BBBBBBBBBB_____
+        _BBBBBBBBBB_____
+        ________________
+        ________________
+        ________________
+        _BBBB___________
+        _BBBB___________
+        _rrrrrrrrrrrrrr_
+        _rrrrrrrrrrrrrr_
+        ________________
+        ________________
+    ''', '''
+        <style>
+            @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+            @page {
+                margin: 1px;
+                size: 16px 7px;
+            }
+            body {
+                color: red;
+                font-family: weasyprint;
+                font-size: 2px;
+                line-height: 1;
+            }
+            div.split {
+                color: blue;
+                float: right;
+            }
+        </style>
+        <div class="split">aaaaa aaaaa aa</div>
+        bbbbbbb''')
