@@ -1,10 +1,4 @@
-"""
-    weasyprint.tests.test_api
-    -------------------------
-
-    Test the public API.
-
-"""
+"""Test the public API."""
 
 import gzip
 import io
@@ -19,10 +13,10 @@ import py
 import pytest
 from PIL import Image
 from weasyprint import CSS, HTML, __main__, default_url_fetcher
-from weasyprint.document import resolve_links
+from weasyprint.links import resolve_links
 from weasyprint.urls import path2url
 
-from .draw import assert_pixels_equal, parse_pixels
+from .draw import parse_pixels
 from .testing_utils import (
     FakeHTML, assert_no_logs, capture_logs, http_server, resource_filename)
 
@@ -40,7 +34,7 @@ def _test_resource(class_, basename, check, **kwargs):
     check(class_(filename=absolute_path, **kwargs))
     check(class_(url, **kwargs))
     check(class_(guess=url, **kwargs))
-    url = path2url(absolute_filename.encode('utf-8'))
+    url = path2url(absolute_filename.encode())
     check(class_(url=url, **kwargs))
     with open(absolute_filename, 'rb') as fd:
         check(class_(fd, **kwargs))
@@ -57,7 +51,7 @@ def _test_resource(class_, basename, check, **kwargs):
     check(class_(relative_path, **kwargs))
     kwargs.pop('base_url', None)
     check(class_(string=content, base_url=relative_filename, **kwargs))
-    encoding = kwargs.get('encoding') or 'utf8'
+    encoding = kwargs.get('encoding') or 'utf-8'
     check(class_(string=content.decode(encoding),  # unicode
                  base_url=relative_filename, **kwargs))
     with pytest.raises(TypeError):
@@ -134,8 +128,8 @@ def _round_meta(pages):
 def test_html_parsing():
     """Test the constructor for the HTML class."""
     _test_resource(FakeHTML, 'doc1.html', _check_doc1)
-    _test_resource(FakeHTML, 'doc1_UTF-16BE.html', _check_doc1,
-                   encoding='UTF-16BE')
+    _test_resource(
+        FakeHTML, 'doc1_UTF-16BE.html', _check_doc1, encoding='UTF-16BE')
 
     py.path.local(os.path.dirname(__file__)).chdir()
     filename = os.path.join('resources', 'doc1.html')
@@ -144,9 +138,11 @@ def test_html_parsing():
     _test_resource(FakeHTML, 'doc1.html', _check_doc1, base_url=filename)
     _check_doc1(FakeHTML(string=string, base_url=filename))
     _check_doc1(FakeHTML(string=string), has_base_url=False)
-    string_with_meta = string.replace(
+    string_with_base = string.replace(
         '<meta', '<base href="resources/"><meta')
-    _check_doc1(FakeHTML(string=string_with_meta, base_url='.'))
+    _check_doc1(FakeHTML(string=string_with_base, base_url='.'))
+    string_with_empty_base = string.replace('<meta', '<base><meta')
+    _check_doc1(FakeHTML(string=string_with_empty_base), has_base_url=False)
 
 
 @assert_no_logs
@@ -168,7 +164,8 @@ def test_css_parsing():
     _test_resource(CSS, 'latin1-test.css', check_css, encoding='latin1')
 
 
-def check_png_pattern(png_bytes, x2=False, blank=False, rotated=False):
+def check_png_pattern(assert_pixels_equal, png_bytes, x2=False, blank=False,
+                      rotated=False):
     if blank:
         expected_pixels = '''
             ________
@@ -180,7 +177,6 @@ def check_png_pattern(png_bytes, x2=False, blank=False, rotated=False):
             ________
             ________
         '''
-        size = 8
     elif x2:
         expected_pixels = '''
             ________________
@@ -200,7 +196,6 @@ def check_png_pattern(png_bytes, x2=False, blank=False, rotated=False):
             ________________
             ________________
         '''
-        size = 16
     elif rotated:
         expected_pixels = '''
             ________
@@ -212,7 +207,6 @@ def check_png_pattern(png_bytes, x2=False, blank=False, rotated=False):
             ________
             ________
         '''
-        size = 8
     else:
         expected_pixels = '''
             ________
@@ -224,20 +218,18 @@ def check_png_pattern(png_bytes, x2=False, blank=False, rotated=False):
             ________
             ________
         '''
-        size = 8
     image = Image.open(io.BytesIO(png_bytes))
-    assert_pixels_equal(
-        'api_png', size, size, image.getdata(),
-        parse_pixels(expected_pixels))
+    width, height, pixels = parse_pixels(expected_pixels)
+    assert_pixels_equal(width, height, image.getdata(), pixels)
 
 
 @assert_no_logs
-def test_python_render(tmpdir):
+def test_python_render(assert_pixels_equal, tmpdir):
     """Test rendering with the Python API."""
     base_url = resource_filename('dummy.html')
     html_string = '<body><img src=pattern.png>'
     css_string = '''
-        @page { margin: 2px; size: 8px; background: #fff }
+        @page { margin: 2px; size: 8px }
         body { margin: 0; font-size: 0 }
         img { image-rendering: pixelated }
 
@@ -250,7 +242,7 @@ def test_python_render(tmpdir):
     pdf_bytes = html.write_pdf(stylesheets=[css])
     assert png_bytes.startswith(b'\211PNG\r\n\032\n')
     assert pdf_bytes.startswith(b'%PDF')
-    check_png_pattern(png_bytes)
+    check_png_pattern(assert_pixels_equal, png_bytes)
 
     png_file = _fake_file()
     html.write_png(png_file, stylesheets=[css])
@@ -276,16 +268,16 @@ def test_python_render(tmpdir):
     assert pdf_bytes.startswith(b'%PDF')
 
     x2_png_bytes = html.write_png(stylesheets=[css], resolution=192)
-    check_png_pattern(x2_png_bytes, x2=True)
+    check_png_pattern(assert_pixels_equal, x2_png_bytes, x2=True)
 
     screen_css = CSS(string=css_string, media_type='screen')
     rotated_png_bytes = html.write_png(stylesheets=[screen_css])
-    check_png_pattern(rotated_png_bytes, rotated=True)
+    check_png_pattern(assert_pixels_equal, rotated_png_bytes, rotated=True)
 
     assert FakeHTML(
         string=html_string, base_url=base_url, media_type='screen'
     ).write_png(
-        stylesheets=[io.BytesIO(css_string.encode('utf8'))]
+        stylesheets=[io.BytesIO(css_string.encode())]
     ) == rotated_png_bytes
     assert FakeHTML(
         string=f'<style>{css_string}</style>{html_string}',
@@ -296,7 +288,7 @@ def test_python_render(tmpdir):
 @assert_no_logs
 def test_command_line_render(tmpdir):
     css = b'''
-        @page { margin: 2px; size: 8px; background: #fff }
+        @page { margin: 2px; size: 8px }
         @media screen { img { transform: rotate(-90deg) } }
         body { margin: 0; font-size: 0 }
     '''
@@ -320,7 +312,7 @@ def test_command_line_render(tmpdir):
     tmpdir.join('no_css.html').write_binary(html)
     tmpdir.join('combined.html').write_binary(combined)
     tmpdir.join('combined-UTF-16BE.html').write_binary(
-        combined.decode('ascii').encode('UTF-16BE'))
+        combined.decode().encode('UTF-16BE'))
     tmpdir.join('linked.html').write_binary(linked)
     tmpdir.join('not_optimized.html').write_binary(not_optimized)
     tmpdir.join('style.css').write_binary(css)
@@ -331,10 +323,10 @@ def test_command_line_render(tmpdir):
     _run('combined-UTF-16BE.html out3.pdf --encoding UTF-16BE')
     assert tmpdir.join('out3.pdf').read_binary() == pdf_bytes
 
-    _run(tmpdir.join('combined.html').strpath + ' out4.pdf')
+    _run(f'{tmpdir.join("combined.html").strpath} out4.pdf')
     assert tmpdir.join('out4.pdf').read_binary() == pdf_bytes
 
-    _run(path2url(tmpdir.join('combined.html').strpath) + ' out5.pdf')
+    _run(f'{path2url(tmpdir.join("combined.html").strpath)} out5.pdf')
     assert tmpdir.join('out5.pdf').read_binary() == pdf_bytes
 
     _run('linked.html --debug out6.pdf')  # test relative URLs
@@ -366,6 +358,7 @@ def test_command_line_render(tmpdir):
     assert tmpdir.join('out13.pdf').read_binary() == rotated_pdf_bytes
     assert tmpdir.join('out14.pdf').read_binary() == rotated_pdf_bytes
 
+    os.environ['SOURCE_DATE_EPOCH'] = '0'
     _run('not_optimized.html out15.pdf')
     _run('not_optimized.html out16.pdf -O images')
     _run('not_optimized.html out17.pdf -O fonts')
@@ -374,12 +367,20 @@ def test_command_line_render(tmpdir):
     _run('not_optimized.html out20.pdf -O none')
     _run('not_optimized.html out21.pdf -O none -O all')
     _run('not_optimized.html out22.pdf -O all -O none')
-    # TODO: test that equivalent CLI options give equivalent PDF sizes,
-    # unfortunately font optimization makes PDF generation not reproducible
     assert (
         len(tmpdir.join('out16.pdf').read_binary()) <
         len(tmpdir.join('out15.pdf').read_binary()) <
         len(tmpdir.join('out20.pdf').read_binary()))
+    assert len({
+        tmpdir.join(f'out{i}.pdf').read_binary()
+        for i in (16, 18, 19, 21)}) == 1
+    assert len({
+        tmpdir.join(f'out{i}.pdf').read_binary()
+        for i in (15, 17)}) == 1
+    assert len({
+        tmpdir.join(f'out{i}.pdf').read_binary()
+        for i in (20, 22)}) == 1
+    os.environ.pop('SOURCE_DATE_EPOCH')
 
     stdout = _run('combined.html -')
     assert stdout.count(b'attachment') == 0
@@ -415,20 +416,71 @@ def test_command_line_render(tmpdir):
         _run('--version')
 
 
+@pytest.mark.parametrize('version, pdf_version', (
+    (1, '1.4'),
+    (2, '1.7'),
+    (3, '1.7'),
+    (4, '2.0'),
+))
+def test_pdfa(version, pdf_version):
+    stdout = _run(f'--pdf-variant=pdf/a-{version}b - -', b'test')
+    assert f'PDF-{pdf_version}'.encode() in stdout
+    assert f'part="{version}"'.encode() in stdout
+
+
+def test_pdf_identifier():
+    stdout = _run('--pdf-identifier=abc - -', b'test')
+    assert b'abc' in stdout
+
+
+def test_pdf_version():
+    stdout = _run('--pdf-version=1.4 - -', b'test')
+    assert b'PDF-1.4' in stdout
+
+
+def test_pdf_custom_metadata():
+    stdout = _run('--custom-metadata - -', b'<meta name=key content=value />')
+    assert b'/key' in stdout
+    assert b'value' in stdout
+
+
+def test_bad_pdf_custom_metadata():
+    stdout = _run(
+        '--custom-metadata - -',
+        '<meta name=é content=value />'.encode('latin1'))
+    assert b'value' not in stdout
+
+
+def test_partial_pdf_custom_metadata():
+    stdout = _run(
+        '--custom-metadata - -',
+        '<meta name=a.b/céd0 content=value />'.encode('latin1'))
+    assert b'/abcd0' in stdout
+    assert b'value' in stdout
+
+
+def test_reproducible():
+    os.environ['SOURCE_DATE_EPOCH'] = '0'
+    stdout1 = _run('- -', b'<body>a<img src=pattern.png>')
+    stdout2 = _run('- -', b'<body>a<img src=pattern.png>')
+    os.environ.pop('SOURCE_DATE_EPOCH')
+    assert stdout1 == stdout2
+
+
 @assert_no_logs
-def test_unicode_filenames(tmpdir):
+def test_unicode_filenames(assert_pixels_equal, tmpdir):
     """Test non-ASCII filenames both in Unicode or bytes form."""
     # Replicate pattern.png in CSS so that base_url does not matter.
     html = b'''
         <style>
-            @page { margin: 2px; size: 8px; background: #fff }
+            @page { margin: 2px; size: 8px }
             html { background: #00f; }
             body { background: #f00; width: 1px; height: 1px }
         </style>
         <body>
     '''
     png_bytes = FakeHTML(string=html).write_png()
-    check_png_pattern(png_bytes)
+    check_png_pattern(assert_pixels_equal, png_bytes)
     unicode_filename = 'Unicödé'
     if sys.platform.startswith('darwin'):  # pragma: no cover
         unicode_filename = unicodedata.normalize('NFD', unicode_filename)
@@ -449,10 +501,10 @@ def test_unicode_filenames(tmpdir):
 
 
 @assert_no_logs
-def test_low_level_api():
+def test_low_level_api(assert_pixels_equal):
     html = FakeHTML(string='<body>')
     css = CSS(string='''
-        @page { margin: 2px; size: 8px; background: #fff }
+        @page { margin: 2px; size: 8px }
         html { background: #00f; }
         body { background: #f00; width: 1px; height: 1px }
     ''')
@@ -471,7 +523,7 @@ def test_low_level_api():
     page, = document.pages
     assert (page.width, page.height) == (8, 8)
     png_bytes = document.write_png(resolution=192)
-    check_png_pattern(png_bytes, x2=True)
+    check_png_pattern(assert_pixels_equal, png_bytes, x2=True)
 
     document = html.render([css])
     page, = document.pages
@@ -642,6 +694,12 @@ def test_low_level_api():
         <body style="transform-origin: 0 0; transform: rotate(90deg)">
         <h1 style="transform: translateX(50px)">!
     ''', [[(1, '!', (0, 50), 'open')]], [('!', (0, 0, 50), [], 'open')], True),
+    ('''
+        <body>
+        <h1 style="width: 10px; line-height: 10px;
+                   transform: skew(45deg, 45deg)">!
+    ''', [[(1, '!', (-5, -5), 'open')]], [('!', (0, -5, -5), [], 'open')],
+     False),
 ))
 @assert_no_logs
 def test_assert_bookmarks(html, expected_by_page, expected_tree, round):
@@ -652,24 +710,25 @@ def test_assert_bookmarks(html, expected_by_page, expected_tree, round):
     assert document.make_bookmark_tree() == expected_tree
 
 
-@assert_no_logs
-def test_links():
-    def assert_links(html, expected_links_by_page, expected_anchors_by_page,
-                     expected_resolved_links,
-                     base_url=resource_filename('<inline HTML>'),
-                     warnings=(), round=False):
-        with capture_logs() as logs:
-            document = FakeHTML(string=html, base_url=base_url).render()
-            if round:
-                _round_meta(document.pages)
-            resolved_links = list(resolve_links(document.pages))
-        assert len(logs) == len(warnings)
-        for message, expected in zip(logs, warnings):
-            assert expected in message
-        assert [p.links for p in document.pages] == expected_links_by_page
-        assert [p.anchors for p in document.pages] == expected_anchors_by_page
-        assert resolved_links == expected_resolved_links
+def assert_links(html, expected_links_by_page, expected_anchors_by_page,
+                 expected_resolved_links,
+                 base_url=resource_filename('<inline HTML>'), warnings=(),
+                 round=False):
+    with capture_logs() as logs:
+        document = FakeHTML(string=html, base_url=base_url).render()
+        if round:
+            _round_meta(document.pages)
+        resolved_links = list(resolve_links(document.pages))
+    assert len(logs) == len(warnings)
+    for message, expected in zip(logs, warnings):
+        assert expected in message
+    assert [p.links for p in document.pages] == expected_links_by_page
+    assert [p.anchors for p in document.pages] == expected_anchors_by_page
+    assert resolved_links == expected_resolved_links
 
+
+@assert_no_logs
+def test_links_1():
     assert_links('''
         <style>
             body { font-size: 10px; line-height: 2; width: 200px }
@@ -713,6 +772,9 @@ def test_links():
             [('lipsum', 0, 0)]),
     ])
 
+
+@assert_no_logs
+def test_links_2():
     assert_links(
         '''
             <body style="width: 200px">
@@ -723,6 +785,9 @@ def test_links():
                   (5, 10, 195, 10), None)], [])],
         base_url='http://weasyprint.org/foo/bar/')
 
+
+@assert_no_logs
+def test_links_3():
     assert_links(
         '''
             <body style="width: 200px">
@@ -734,6 +799,9 @@ def test_links():
                   (5, 10, 195, 10), None)], [])],
         base_url='http://weasyprint.org/foo/bar/')
 
+
+@assert_no_logs
+def test_links_4():
     # Relative URI reference without a base URI: allowed for links
     assert_links(
         '''
@@ -743,6 +811,9 @@ def test_links():
         [([('external', '../lipsum', (5, 10, 195, 10), None)], [])],
         base_url=None)
 
+
+@assert_no_logs
+def test_links_5():
     # Relative URI reference without a base URI: not supported for -weasy-link
     assert_links(
         '''
@@ -753,6 +824,9 @@ def test_links():
             'WARNING: Ignored `-weasy-link: url(../lipsum)` at 1:1, '
             'Relative URI reference without a base URI'])
 
+
+@assert_no_logs
+def test_links_6():
     # Internal or absolute URI reference without a base URI: OK
     assert_links(
         '''
@@ -769,6 +843,9 @@ def test_links():
           [('lipsum', 5, 10)])],
         base_url=None)
 
+
+@assert_no_logs
+def test_links_7():
     assert_links(
         '''
             <body style="width: 200px">
@@ -781,6 +858,9 @@ def test_links():
           [('lipsum', 5, 10)])],
         base_url=None)
 
+
+@assert_no_logs
+def test_links_8():
     assert_links(
         '''
             <style> a { display: block; height: 15px } </style>
@@ -797,6 +877,9 @@ def test_links():
         warnings=[
             'ERROR: No anchor #missing for internal URI reference'])
 
+
+@assert_no_logs
+def test_links_9():
     assert_links(
         '''
             <body style="width: 100px; transform: translateY(100px)">
@@ -809,6 +892,9 @@ def test_links():
           [('lipsum', 70, 10)])],
         round=True)
 
+
+@assert_no_logs
+def test_links_10():
     # Download for attachment
     assert_links(
         '''
@@ -822,12 +908,23 @@ def test_links():
         base_url=None)
 
 
+@assert_no_logs
+def test_links_11():
+    # Attachment with missing href
+    assert_links(
+        '''
+            <body style="width: 200px">
+            <a rel=attachment download="wow.png"
+                style="display: block; margin: 10px 5px">
+        ''', [[]], [{}], [([], [])], base_url=None)
+
+
 # Make relative URL references work with our custom URL scheme.
 uses_relative.append('weasyprint-custom')
 
 
 @assert_no_logs
-def test_url_fetcher():
+def test_url_fetcher(assert_pixels_equal):
     filename = resource_filename('pattern.png')
     with open(filename, 'rb') as pattern_fd:
         pattern_png = pattern_fd.read()
@@ -848,13 +945,15 @@ def test_url_fetcher():
 
     base_url = resource_filename('dummy.html')
     css = CSS(string='''
-        @page { size: 8px; margin: 2px; background: #fff }
+        @page { size: 8px; margin: 2px }
         body { margin: 0; font-size: 0 }
     ''', base_url=base_url)
 
     def test(html, blank=False):
         html = FakeHTML(string=html, url_fetcher=fetcher, base_url=base_url)
-        check_png_pattern(html.write_png(stylesheets=[css]), blank=blank)
+        check_png_pattern(
+            assert_pixels_equal, html.write_png(stylesheets=[css]),
+            blank=blank)
 
     test('<body><img src="pattern.png">')  # Test a "normal" URL
     test(f'<body><img src="{Path(filename).as_uri()}">')
@@ -890,20 +989,26 @@ def test_url_fetcher():
         url_fetcher=fetcher_2).render()
 
 
-@assert_no_logs
-def test_html_meta():
-    def assert_meta(html, **meta):
-        meta.setdefault('title', None)
-        meta.setdefault('authors', [])
-        meta.setdefault('keywords', [])
-        meta.setdefault('generator', None)
-        meta.setdefault('description', None)
-        meta.setdefault('created', None)
-        meta.setdefault('modified', None)
-        meta.setdefault('attachments', [])
-        assert vars(FakeHTML(string=html).render().metadata) == meta
+def assert_meta(html, **meta):
+    meta.setdefault('title', None)
+    meta.setdefault('authors', [])
+    meta.setdefault('keywords', [])
+    meta.setdefault('generator', None)
+    meta.setdefault('description', None)
+    meta.setdefault('created', None)
+    meta.setdefault('modified', None)
+    meta.setdefault('attachments', [])
+    meta.setdefault('custom', {})
+    assert vars(FakeHTML(string=html).render().metadata) == meta
 
+
+@assert_no_logs
+def test_html_meta_1():
     assert_meta('<body>')
+
+
+@assert_no_logs
+def test_html_meta_2():
     assert_meta(
         '''
             <meta name=author content="I Me &amp; Myself">
@@ -929,7 +1034,12 @@ def test_html_meta():
         keywords=['html', 'css', 'pdf', 'Python; pydyf'],
         description="Blah… ",
         created='2011-04',
-        modified='2013')
+        modified='2013',
+        custom={'dummy': 'ignored'})
+
+
+@assert_no_logs
+def test_html_meta_3():
     assert_meta(
         '''
             <title>One</title>
@@ -942,6 +1052,21 @@ def test_html_meta():
         authors=['', 'Me'])
 
 
+@assert_no_logs
+def test_html_meta_4():
+    with capture_logs() as logs:
+        assert_meta(
+            '''
+                <meta name=dcterms.created content=wrong>
+                <meta name=author content=Me>
+                <title>Title</title>
+            ''',
+            title='Title',
+            authors=['Me'])
+    assert len(logs) == 1
+    assert 'Invalid date' in logs[0]
+
+
 @assert_no_logs
 def test_http():
     def gzip_compress(data):
@@ -971,10 +1096,10 @@ def test_http():
             (b'<html test=accept-encoding-header-fail>', [])
         ),
     }) as root_url:
-        assert HTML(root_url + '/gzip').etree_element.get('test') == 'ok'
-        assert HTML(root_url + '/deflate').etree_element.get('test') == 'ok'
+        assert HTML(f'{root_url}/gzip').etree_element.get('test') == 'ok'
+        assert HTML(f'{root_url}/deflate').etree_element.get('test') == 'ok'
         assert HTML(
-            root_url + '/raw-deflate').etree_element.get('test') == 'ok'
+            f'{root_url}/raw-deflate').etree_element.get('test') == 'ok'
 
 
 @assert_no_logs
