@@ -1,16 +1,14 @@
-"""
-    weasyprint.css
-    --------------
+"""Find and apply CSS.
 
-    This module takes care of steps 3 and 4 of “CSS 2.1 processing model”:
-    Retrieve stylesheets associated with a document and annotate every element
-    with a value for every CSS property.
+This module takes care of steps 3 and 4 of “CSS 2.1 processing model”: Retrieve
+stylesheets associated with a document and annotate every element with a value
+for every CSS property.
 
-    http://www.w3.org/TR/CSS21/intro.html#processing-model
+http://www.w3.org/TR/CSS21/intro.html#processing-model
 
-    This module does this in more than two steps. The
-    :func:`get_all_computed_styles` function does everything, but it is itsef
-    based on other functions in this module.
+This module does this in more than two steps. The
+:func:`get_all_computed_styles` function does everything, but it is itsef based
+on other functions in this module.
 
 """
 
@@ -35,7 +33,6 @@ PSEUDO_ELEMENTS = (
     None, 'before', 'after', 'marker', 'first-line', 'first-letter',
     'footnote-call', 'footnote-marker')
 
-
 PageType = namedtuple('PageType', ['side', 'blank', 'first', 'index', 'name'])
 
 
@@ -214,12 +211,11 @@ class StyleFor:
         if selector_page_type.index is not None:
             a, b, group = selector_page_type.index
             # TODO: handle group
-            if a:
-                if (page_type.index + 1 - b) % a:
-                    return False
+            offset = page_type.index + 1 - b
+            if a == 0:
+                return offset == 0
             else:
-                if page_type.index + 1 != b:
-                    return False
+                return offset / a >= 0 and not offset % a
         return True
 
 
@@ -232,6 +228,23 @@ def get_child_text(element):
     return ''.join(content)
 
 
+def text_decoration(key, value, parent_value, cascaded):
+    # The text-decoration-* properties are not inherited but propagated
+    # using specific rules.
+    # See https://drafts.csswg.org/css-text-decor-3/#line-decoration
+    # TODO: these rules don’t follow the specification.
+    if key in ('text_decoration_color', 'text_decoration_style'):
+        if not cascaded:
+            value = parent_value
+    elif key == 'text_decoration_line':
+        if parent_value != 'none':
+            if value == 'none':
+                value = parent_value
+            else:
+                value = value | parent_value
+    return value
+
+
 def find_stylesheets(wrapper_element, device_media_type, url_fetcher, base_url,
                      font_config, counter_style, page_rules):
     """Yield the stylesheets in ``element_tree``.
@@ -587,15 +600,7 @@ def declaration_precedence(origin, importance):
         return 5
 
 
-class Style(dict):
-    @property
-    def variables(self):
-        return {
-            key: value for (key, value) in self.items()
-            if key.startswith('__')}
-
-
-class AnonymousStyle(Style):
+class AnonymousStyle(dict):
     """Computed style used for anonymous boxes."""
     def __init__(self, parent_style):
         # border-*-style is none, so border-width computes to zero.
@@ -611,7 +616,9 @@ class AnonymousStyle(Style):
         self.parent_style = parent_style
         self.specified = self
         if parent_style:
-            self.update(parent_style.variables)
+            self.cache = parent_style.cache
+        else:
+            self.cache = {'ratio_ch': {}, 'ratio_ex': {}}
 
     def copy(self):
         copy = AnonymousStyle(self.parent_style)
@@ -624,12 +631,16 @@ class AnonymousStyle(Style):
         elif key == 'page':
             # page is not inherited but taken from the ancestor if 'auto'
             self[key] = self.parent_style[key]
+        elif key.startswith('text_decoration_'):
+            self[key] = text_decoration(
+                key, INITIAL_VALUES[key], self.parent_style[key],
+                cascaded=False)
         else:
             self[key] = INITIAL_VALUES[key]
         return self[key]
 
 
-class ComputedStyle(Style):
+class ComputedStyle(dict):
     """Computed style used for non-anonymous boxes."""
     def __init__(self, parent_style, cascaded, element, pseudo_type,
                  root_style, base_url):
@@ -642,10 +653,9 @@ class ComputedStyle(Style):
         self.root_style = root_style
         self.base_url = base_url
         if parent_style:
-            self.update(parent_style.variables)
-        self.update({
-            key: value[0] for (key, value) in cascaded.items()
-            if key.startswith('__')})
+            self.cache = parent_style.cache
+        else:
+            self.cache = {'ratio_ch': {}, 'ratio_ex': {}}
 
     def copy(self):
         copy = ComputedStyle(
@@ -666,14 +676,17 @@ class ComputedStyle(Style):
         if key in self.cascaded:
             value = keyword = self.cascaded[key][0]
         else:
-            keyword = 'inherit' if key in INHERITED else 'initial'
+            if key in INHERITED or key.startswith('__'):
+                keyword = 'inherit'
+            else:
+                keyword = 'initial'
 
         if keyword == 'inherit' and self.parent_style is None:
             # On the root element, 'inherit' from initial values
             keyword = 'initial'
 
         if keyword == 'initial':
-            value = INITIAL_VALUES[key]
+            value = None if key.startswith('__') else INITIAL_VALUES[key]
             if key not in INITIAL_NOT_COMPUTED:
                 # The value is the same as when computed
                 self[key] = value
@@ -681,6 +694,10 @@ class ComputedStyle(Style):
             # Values in parent_style are already computed.
             self[key] = value = self.parent_style[key]
 
+        if key.startswith('text_decoration_') and self.parent_style:
+            self[key] = text_decoration(
+                key, value, self.parent_style[key], key in self.cascaded)
+
         if key == 'page' and value == 'auto':
             # The page property does not inherit. However, if the page
             # value on an element is auto, then its used value is the value
@@ -696,8 +713,7 @@ class ComputedStyle(Style):
         if key in self:
             return self[key]
 
-        getter = computed_values.COMPUTER_FUNCTIONS.get
-        function = getter(key)
+        function = computed_values.COMPUTER_FUNCTIONS.get(key)
         already_computed_value = False
 
         if value:
@@ -857,12 +873,9 @@ def parse_page_selectors(rule):
 
 
 def preprocess_stylesheet(device_media_type, base_url, stylesheet_rules,
-                          url_fetcher, matcher, page_rules, fonts,
-                          font_config, counter_style, ignore_imports=False):
-    """Do the work that can be done early on stylesheet, before they are
-    in a document.
-
-    """
+                          url_fetcher, matcher, page_rules, font_config,
+                          counter_style, ignore_imports=False):
+    """Do what can be done early on stylesheet, before being in a document."""
     for rule in stylesheet_rules:
         if getattr(rule, 'content', None) is None and (
                 rule.type != 'at-rule' or rule.lower_at_keyword != 'import'):
@@ -958,7 +971,7 @@ def preprocess_stylesheet(device_media_type, base_url, stylesheet_rules,
             content_rules = tinycss2.parse_rule_list(rule.content)
             preprocess_stylesheet(
                 device_media_type, base_url, content_rules, url_fetcher,
-                matcher, page_rules, fonts, font_config, counter_style,
+                matcher, page_rules, font_config, counter_style,
                 ignore_imports=True)
 
         elif rule.type == 'at-rule' and rule.lower_at_keyword == 'page':
@@ -1011,10 +1024,7 @@ def preprocess_stylesheet(device_media_type, base_url, stylesheet_rules,
                     break
             else:
                 if font_config is not None:
-                    font_filename = font_config.add_font_face(
-                        rule_descriptors, url_fetcher)
-                    if font_filename:
-                        fonts.append(font_filename)
+                    font_config.add_font_face(rule_descriptors, url_fetcher)
 
         elif (rule.type == 'at-rule' and
                 rule.lower_at_keyword == 'counter-style'):
@@ -1041,10 +1051,10 @@ def preprocess_stylesheet(device_media_type, base_url, stylesheet_rules,
                 'symbols': None,
                 'additive_symbols': None,
             }
-            rule_descriptors = dict(
-                preprocess_descriptors('counter-style', base_url, content))
+            rule_descriptors = preprocess_descriptors(
+                'counter-style', base_url, content)
 
-            for descriptor_name, descriptor_value in rule_descriptors.items():
+            for descriptor_name, descriptor_value in rule_descriptors:
                 counter[descriptor_name] = descriptor_value
 
             if counter['system'] is None:
