@@ -1,15 +1,10 @@
-"""
-    weasyprint.layout.inline
-    ------------------------
-
-    Line breaking and layout for inline-level boxes.
-
-"""
+"""Layout for inline-level boxes."""
 
 import unicodedata
+from math import inf
 
 from ..css import computed_from_cascaded
-from ..css.computed_values import ex_ratio, strut_layout
+from ..css.computed_values import character_ratio, strut_layout
 from ..formatting_structure import boxes
 from ..text.line_break import can_break_text, create_layout, split_first_line
 from .absolute import AbsolutePlaceholder, absolute_layout
@@ -192,7 +187,6 @@ def skip_first_whitespace(box, skip_stack):
         index = 0
         next_skip_stack = None
     else:
-        # TODO: handle multiple skip stacks
         (index, next_skip_stack), = skip_stack.items()
 
     if isinstance(box, boxes.TextBox):
@@ -303,7 +297,6 @@ def first_letter_to_box(box, skip_stack, first_letter_style):
             elif child.text:
                 character_found = False
                 if skip_stack:
-                    # TODO: handle multiple skip stacks
                     child_skip_stack, = skip_stack.values()
                     if child_skip_stack:
                         index, = child_skip_stack
@@ -353,7 +346,6 @@ def first_letter_to_box(box, skip_stack, first_letter_style):
                             index: {child_index + 1: grandchild_skip_stack}}
         elif isinstance(child, boxes.ParentBox):
             if skip_stack:
-                # TODO: handle multiple skip stacks
                 child_skip_stack, = skip_stack.values()
             else:
                 child_skip_stack = None
@@ -405,10 +397,11 @@ def inline_block_box_layout(context, box, position_x, skip_stack,
 
     box.position_x = position_x
     box.position_y = 0
-    box, _, _, _, _ = block_container_layout(
-        context, box, bottom_space=-float('inf'), skip_stack=skip_stack,
+    box, _, _, _, _, _ = block_container_layout(
+        context, box, bottom_space=-inf, skip_stack=skip_stack,
         page_is_empty=True, absolute_boxes=absolute_boxes,
-        fixed_boxes=fixed_boxes, adjoining_margins=None, discard=False)
+        fixed_boxes=fixed_boxes, adjoining_margins=None, discard=False,
+        max_lines=None)
     box.baseline = inline_block_baseline(box)
     return box
 
@@ -472,8 +465,10 @@ def split_inline_level(context, box, position_x, max_x, bottom_space,
             skip = skip or 0
             assert skip_stack is None
 
+        is_line_start = len(line_children) == 0
         new_box, skip, preserved_line_break = split_text_box(
-            context, box, max_x - position_x, skip)
+            context, box, max_x - position_x, skip,
+            is_line_start=is_line_start)
 
         if skip is None:
             resume_at = None
@@ -515,8 +510,8 @@ def split_inline_level(context, box, position_x, max_x, bottom_space,
             if getattr(box, f'margin_{side}') == 'auto':
                 setattr(box, f'margin_{side}', 0)
         new_box, resume_at, _, _, _ = flex_layout(
-            context, box, -float('inf'), skip_stack, containing_block,
-            False, absolute_boxes, fixed_boxes)
+            context, box, -inf, skip_stack, containing_block, False,
+            absolute_boxes, fixed_boxes)
         preserved_line_break = False
         first_letter = '\u2e80'
         last_letter = '\u2e80'
@@ -536,7 +531,7 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
         child.position_x = position_x
         placeholder = AbsolutePlaceholder(child)
         line_placeholders.append(placeholder)
-        waiting_children.append((index, placeholder))
+        waiting_children.append((index, placeholder, child))
         if child.style['position'] == 'absolute':
             absolute_boxes.append(placeholder)
         else:
@@ -549,7 +544,7 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
         # To retrieve the real available space for floats, we must remove
         # the trailing whitespaces from the line
         non_floating_children = [
-            child_ for _, child_ in (children + waiting_children)
+            child_ for _, child_, _ in (children + waiting_children)
             if not child_.is_floated()]
         if non_floating_children:
             float_width -= trailing_whitespace_size(
@@ -566,8 +561,8 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
             if float_resume_at:
                 context.broken_out_of_flow.append(
                     (child, containing_block, float_resume_at))
+            waiting_children.append((index, new_child, child))
             child = new_child
-            waiting_children.append((index, child))
 
             # Translate previous line children
             dx = max(child.margin_width(), 0)
@@ -608,60 +603,41 @@ def _break_waiting_children(context, box, max_x, bottom_space,
         # fixed_boxes and other lists.
         waiting_children_copy = waiting_children.copy()
         while waiting_children_copy:
-            child_index, child = waiting_children_copy.pop()
-            if child.is_in_normal_flow() and can_break_inside(child):
-                # Break the waiting child at its last possible breaking point.
-                # TODO: The dirty solution chosen here is to decrease the
-                # actual size by 1 and render the waiting child again with this
-                # constraint. We may find a better way.
-                max_x = child.position_x + child.margin_width() - 1
+            child_index, child, original_child = waiting_children_copy.pop()
+            if not child.is_in_normal_flow() or not can_break_inside(child):
+                continue
+
+            if initial_skip_stack and child_index in initial_skip_stack:
+                child_skip_stack = initial_skip_stack[child_index]
+            else:
+                child_skip_stack = None
+
+            # Break the waiting child at its last possible breaking point.
+            # TODO: The dirty solution chosen here is to decrease the
+            # actual size by 1 and render the waiting child again with this
+            # constraint. We may find a better way.
+            max_x = child.position_x + child.margin_width() - 1
+            while max_x > child.position_x:
                 new_child, child_resume_at, _, _, _, _ = split_inline_level(
-                    context, child, child.position_x, max_x, bottom_space,
-                    None, box, absolute_boxes, fixed_boxes, line_placeholders,
-                    waiting_floats, line_children)
+                    context, original_child, child.position_x, max_x,
+                    bottom_space, child_skip_stack, box, absolute_boxes,
+                    fixed_boxes, line_placeholders, waiting_floats,
+                    line_children)
+                if child_resume_at:
+                    break
+                max_x -= 1
+            else:
+                # No line break found
+                continue
 
-                children.extend(waiting_children_copy)
-                if new_child is None:
-                    # May be None where we have an empty TextBox.
-                    assert isinstance(child, boxes.TextBox)
-                else:
-                    children.append((child_index, new_child))
+            children.extend(waiting_children_copy)
+            if new_child is None:
+                # May be None where we have an empty TextBox.
+                assert isinstance(child, boxes.TextBox)
+            else:
+                children.append((child_index, new_child, child))
 
-                # As this child has already been broken following the original
-                # skip stack, we have to add the original skip stack to the
-                # partial skip stack we get after the new rendering.
-
-                # Combining skip stacks is a bit complicated. We have to:
-                # - set `child_index` as the first number
-                # - append the new stack if it's an absolute one
-                # - otherwise append the combined stacks
-                #   (resume_at + initial_skip_stack)
-
-                # extract the initial index
-                if initial_skip_stack is None:
-                    current_skip_stack = None
-                    initial_index = 0
-                else:
-                    (initial_index, current_skip_stack), = (
-                        initial_skip_stack.items())
-                # child_resume_at is an absolute skip stack
-                if child_index > initial_index:
-                    return {child_index: child_resume_at}
-
-                # combine the stacks
-                current_resume_at = child_resume_at
-                stack = []
-                while current_skip_stack and current_resume_at:
-                    (skip, current_skip_stack), = current_skip_stack.items()
-                    (resume, current_resume_at), = current_resume_at.items()
-                    stack.append(skip + resume)
-                    if resume != 0:
-                        break
-                resume_at = current_resume_at
-                while stack:
-                    resume_at = {stack.pop(): resume_at}
-                # insert the child index
-                return {child_index: resume_at}
+            return {child_index: child_resume_at}
 
     if children:
         # Too wide, can't break waiting children and the inline is
@@ -705,7 +681,6 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
     if is_start:
         skip = 0
     else:
-        # TODO: handle multiple skip stacks
         (skip, skip_stack), = skip_stack.items()
 
     for i, child in enumerate(box.children[skip:]):
@@ -785,10 +760,10 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
         else:
             if isinstance(box, boxes.LineBox):
                 line_children.append((index, new_child))
-            # TODO: we should try to find a better condition here.
             trailing_whitespace = (
                 isinstance(new_child, boxes.TextBox) and
-                not new_child.text.strip())
+                new_child.text and
+                unicodedata.category(new_child.text[-1]) == 'Zs')
             new_position_x = new_child.position_x + new_child.margin_width()
 
             if new_position_x > max_x and not trailing_whitespace:
@@ -801,7 +776,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
                     break
 
             position_x = new_position_x
-            waiting_children.append((index, new_child))
+            waiting_children.append((index, new_child, child))
 
         waiting_floats.extend(child_waiting_floats)
         if resume_at is not None:
@@ -815,7 +790,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
     # Reorder inline blocks when direction is rtl
     if box.style['direction'] == 'rtl' and len(children) > 1:
         in_flow_children = [
-            box_child for _, box_child in children
+            box_child for _, box_child, _ in children
             if box_child.is_in_normal_flow()]
         position_x = in_flow_children[0].position_x
         for child in in_flow_children[::-1]:
@@ -825,7 +800,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
 
     is_end = resume_at is None
     new_box = box.copy_with_children(
-        [box_child for index, box_child in children])
+        [box_child for index, box_child, _ in children])
     new_box.remove_decoration(start=not is_start, end=not is_end)
     if isinstance(box, boxes.LineBox):
         # We must reset line box width according to its new children
@@ -882,7 +857,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
         float_widths)
 
 
-def split_text_box(context, box, available_width, skip):
+def split_text_box(context, box, available_width, skip, is_line_start=True):
     """Keep as much text as possible from a TextBox in a limited width.
 
     Try not to overflow but always have some text in ``new_box``.
@@ -900,7 +875,8 @@ def split_text_box(context, box, available_width, skip):
     if font_size == 0 or not text:
         return None, None, False
     layout, length, resume_index, width, height, baseline = split_first_line(
-        text, box.style, context, available_width, box.justification_spacing)
+        text, box.style, context, available_width, box.justification_spacing,
+        is_line_start=is_line_start)
     assert resume_index != 0
 
     # Convert ``length`` and ``resume_at`` from UTF-8 indexes in text
@@ -909,11 +885,11 @@ def split_text_box(context, box, available_width, skip):
     # resume_at is not set). One code point is one or more byte, so
     # UTF-8 indexes are always bigger or equal to Unicode indexes.
     new_text = layout.text
-    encoded = text.encode('utf8')
+    encoded = text.encode()
     if resume_index is not None:
-        between = encoded[length:resume_index].decode('utf8')
-        resume_index = len(encoded[:resume_index].decode('utf8'))
-    length = len(encoded[:length].decode('utf8'))
+        between = encoded[length:resume_index].decode()
+        resume_index = len(encoded[:resume_index].decode())
+    length = len(encoded[:length].decode())
 
     if length > 0:
         box = box.copy_with_text(new_text)
@@ -1045,8 +1021,8 @@ def inline_box_verticality(box, top_bottom_subtrees, baseline_y):
         if vertical_align == 'baseline':
             child_baseline_y = baseline_y
         elif vertical_align == 'middle':
-            one_ex = box.style['font_size'] * ex_ratio(box.style)
-            top = baseline_y - (one_ex + child.margin_height()) / 2.
+            one_ex = box.style['font_size'] * character_ratio(box.style, 'x')
+            top = baseline_y - (one_ex + child.margin_height()) / 2
             child_baseline_y = top + child.baseline
         elif vertical_align == 'text-top':
             # align top with the top of the parent’s content area
@@ -1161,8 +1137,8 @@ def add_word_spacing(context, box, justification_spacing, x_advance):
         nb_spaces = count_spaces(box)
         if nb_spaces > 0:
             layout = create_layout(
-                box.text, box.style, context, -float('inf'),
-                box.justification_spacing)
+                box.text, box.style, context, max_width=None,
+                justification_spacing=box.justification_spacing)
             layout.deactivate()
             extra_space = justification_spacing * nb_spaces
             x_advance += extra_space
